#include <reg51.h>
#include <stdio.h>

//MakeWiz -> Components Tab -> Add Source File -> ENG_I2C.s51
#include "ENG_I2C.h"

//Hubungan DT-51 LCMS dengan SPC CHARACTER LCD (I2C)
#asm
  .export SDA,SCL
  SDA=P3.2            //Pin SDA = P3.2
  SCL=P3.3            //Pin SCL = P3.3
#endasm

near unsigned char AddressI2C;

//Text disimpan pada code memory (\x00 = Null, 00H)
code char text1[]={"DT-51  I2C\x00"};
code char text2[]={"SPC Serial LCD\x00"};

#define ADDR_SPC_LCD    0xEE    //Address SPC CHARACTER LCD (I2C)

void K10_Control (unsigned char addr, unsigned char control)
{
    i2c_sendstart();
    i2c_setbyte(addr);          //Kirim address
    i2c_setbyte(0x00);          //Kirim Command Control
    i2c_setbyte(control);       //Kirim Data Control
    i2c_sendstop();
    _wait_ms(5);
}

void K10_WriteSingle (unsigned char addr, unsigned char character)
{
    i2c_sendstart();
    i2c_setbyte(addr);          //Kirim address
    i2c_setbyte(0x10);          //Kirim Command Write Single Character
    i2c_setbyte(character);     //Kirim Data Character
    i2c_sendstop();
    _wait_ms(5);
}

unsigned char K10_WS_Next (unsigned char addr, unsigned char i)
{
    i=0;
    i2c_sendstop();
    _wait_ms(8);
    i2c_sendstart();
    i2c_setbyte(addr);          //Kirim address
    i2c_setbyte(0x20);          //Kirim Command Write Multiple Character
    return i;
}

void K10_WriteString (unsigned char addr, unsigned int *ptr)
{
    unsigned char i,dat;
    i2c_sendstart();
    i2c_setbyte(addr);          //Kirim address
    i2c_setbyte(0x20);          //Kirim Command Write Multiple Character
    i=0;
    dat=(*ptr>>8);
    while(dat!=0)
      {
        i2c_setbyte(dat);       //Data Character (High byte)
        dat=*ptr;
        i++;
        if (i==7) i=K10_WS_Next(addr,i);
        if (dat!=0)
          {
            i2c_setbyte(dat);   //Data Character (Low byte)
            ptr++;
            dat=(*ptr>>8);
            i++;
            if (i==7) i=K10_WS_Next(addr,i);
          }
      }
    i2c_setbyte(dat);           //Null character
    i2c_sendstop();
    _wait_ms(5);
}

void K10_LocateDDRAM (unsigned char addr, unsigned char kolom, unsigned char baris)
{
    i2c_sendstart();
    i2c_setbyte(addr);          //Kirim address
    i2c_setbyte(0x30);          //Kirim Command Locate DDRAM
    i2c_setbyte(kolom);         //Kirim Data Kolom
    i2c_setbyte(baris);         //Kirim Data Baris
    i2c_sendstop();
    _wait_ms(5);
}

void K10_LocateCGRAM (unsigned char addr, unsigned char alamat)
{
    i2c_sendstart();
    i2c_setbyte(addr);          //Kirim address
    i2c_setbyte(0x40);          //Kirim Command Locate CGRAM
    i2c_setbyte(alamat);        //Kirim Data Alamat
    i2c_sendstop();
    _wait_ms(5);
}

void K10_SetContrast (unsigned char addr, unsigned char contrast)
{
    i2c_sendstart();
    i2c_setbyte(addr);          //Kirim address
    i2c_setbyte(0x50);          //Kirim Command Set Contrast
    i2c_setbyte(contrast);      //Kirim Data Contrast
    i2c_sendstop();
    _wait_ms(5);
}

unsigned char K10_ReadDDRAMData (unsigned char addr)
{
    unsigned char dat;
    i2c_sendstart();
    i2c_setbyte(addr);          //Kirim address
    i2c_setbyte(0x60);          //Kirim Command Read Data DDRAM
    i2c_sendstop();
    
    _wait_ms(5);
    i2c_sendstart();
    i2c_setbyte(addr | 0x01);   //Kirim address (I2C read)
    dat=i2c_getbytew();         //Terima Data Character
    i2c_sendstop();
    _wait_ms(5);
    return dat;
}

unsigned char K10_ReadCGRAMData (unsigned char addr)
{
    unsigned char dat;
    i2c_sendstart();
    i2c_setbyte(addr);          //Kirim address
    i2c_setbyte(0x70);          //Kirim Command Read Data CGRAM
    i2c_sendstop();

    _wait_ms(5);
    i2c_sendstart();
    i2c_setbyte(addr | 0x01);   //Kirim address (I2C read)
    dat=i2c_getbytew();        //Terima Data Character
    i2c_sendstop();
    _wait_ms(5);
    return dat;
}

unsigned char K10_ReadDDRAMAddr (unsigned char addr)
{
    unsigned char dat;
    i2c_sendstart();
    i2c_setbyte(addr);          //Kirim address
    i2c_setbyte(0x80);          //Kirim Command Read Address DDRAM
    i2c_sendstop();

    _wait_ms(5);
    i2c_sendstart();
    i2c_setbyte(addr | 0x01);   //Kirim address (I2C read)
    dat=i2c_getbytew();        //Terima Data Character
    i2c_sendstop();
    _wait_ms(5);
    return dat;
}

unsigned char K10_ReadCGRAMAddr (unsigned char addr)
{
    unsigned char dat;
    i2c_sendstart();
    i2c_setbyte(addr);          //Kirim address
    i2c_setbyte(0x90);          //Kirim Command Read Address CGRAM
    i2c_sendstop();

    _wait_ms(5);
    i2c_sendstart();
    i2c_setbyte(addr | 0x01);   //Kirim address (I2C read)
    dat=i2c_getbytew();        //Terima Data Character
    i2c_sendstop();
    _wait_ms(5);
    return dat;
}

void K10_CommandLCD (unsigned char addr, unsigned char command)
{
    i2c_sendstart();
    i2c_setbyte(addr);          //Kirim address
    i2c_setbyte(0xA0);          //Kirim Command LCD
    i2c_setbyte(command);       //Kirim Data Command
    i2c_sendstop();
    _wait_ms(5);
}

void main (void)
{
    unsigned char data;
    IE=0x00;
    AddressI2C=ADDR_SPC_LCD;
	_wait_ms(1000);

    K10_Control(AddressI2C,0x8E);       //BackLight ON, Cursor Shift, No Shift, Increment, Display ON, Cursor ON dan Blink OFF
    K10_SetContrast(AddressI2C,0x01);   //Kontras LCD = 07
    K10_CommandLCD(AddressI2C,0x01);    //Inst.LCD (Display Clear), DDRAM addr = 00H
    _wait_ms(1000);
    
    K10_WriteSingle(AddressI2C,'I');
    K10_WriteSingle(AddressI2C,'E');
    
    K10_LocateDDRAM(AddressI2C,0x00,0x01);  //Copy character kolom 0, baris 1
    data=K10_ReadDDRAMData(AddressI2C);
    K10_LocateDDRAM(AddressI2C,14,0x01);    //Paste character kolom 14, baris 1
    K10_WriteSingle(AddressI2C,data);
    K10_LocateDDRAM(AddressI2C,0x01,0x01);  //Copy character kolom 1, baris 1
    data=K10_ReadDDRAMData(AddressI2C);
    K10_LocateDDRAM(AddressI2C,15,0x01);    //Paste character kolom 15, baris 1
    K10_WriteSingle(AddressI2C,data);

    K10_LocateDDRAM(AddressI2C,0x03,0x01);	//ubah posisi kursor ke kolom 3 baris 1
    K10_WriteString(AddressI2C,text1);		//kirimkan data text1 yang berisi "DT-51 I2C"
    K10_LocateDDRAM(AddressI2C,0x00,0x02);	//ubah posisi kursor ke kolom 0 baris 2
    K10_WriteString(AddressI2C,text2);		//kirimkan data text2 yang berisi "SPC Serial LCD"
		
    K10_LocateDDRAM(AddressI2C,0x00,0x01);  //Kursor pada kolom 0, baris 1
    _wait_ms(1500);
    K10_Control(AddressI2C,0x8F);       //BackLight ON, Cursor Shift, No Shift, Increment, Display ON, Cursor ON dan Blink ON
    _wait_ms(1500);
    K10_Control(AddressI2C,0x0E);       //BackLight OFF, Cursor Shift, No Shift, Increment, Display ON, Cursor ON dan Blink OFF
    _wait_ms(1500);
    K10_Control(AddressI2C,0x8C);       //BackLight ON, Cursor Shift, No Shift, Increment, Display ON, Cursor OFF dan Blink OFF
    while(1);
}