#include <reg51.h>
#include <stdio.h>

//Hubungan DT-51 LCMS dengan SPC CHARACTER LCD (SynSer)
#define CLR     P3_B2   //Pin P3.2 --- SPC Character LCD Pin 1 (SerClr)
#define DATA    P3_B1   //Pin P3.1 --- SPC Character LCD Pin 3 (SerIn)
#define CLK     P3_B0   //Pin P3.0 --- SPC Character LCD Pin 2 (SerClk)

near unsigned char AddressSS;

//Text disimpan pada code memory (\x00 = Null, 00H)
code char text1[]={"DT-51 SySr\x00"};
code char text2[]={"SPC Serial LCD\x00"};

#define ADDR_SPC_LCD    0xE0    //Address SPC CHARACTER LCD (SynSer)

void SynchSerSend (unsigned char dat)
{
    unsigned char i,j;
    CLK=1;
    DATA=1;
    for (i=0; i<8; i++)
      {
        if ((dat & 0x01)==1)  DATA=1;
        else DATA=0;
        CLK=0;
        for (j=0; j<200; j++);
        CLK=1;
        dat>>=1;
      }
    DATA=1;
}

unsigned char SynchSerGet (void)
{
    unsigned char i,j,dat;
    CLK=1;
    DATA=1;
    dat=0;
    for (i=0; i<8; i++)
      {
        dat>>=1;
        CLK=0;
        for (j=0; j<10; j++);
        dat|=(DATA<<7);
        CLK=1;
        DATA=1;
      }
    return dat;
}

void SynchSerReset (void)
{
    unsigned char i;
    CLR=0;
    for (i=0; i<150; i++);
    CLR=1;
}

void K10_Control_SS (unsigned char addr, unsigned char dat)
{
    addr&=0xE0;
    SynchSerSend(addr);     //Command Control
    SynchSerSend(dat);      //Data Control
    _wait_ms(10);
}

void K10_WriteSingle_SS (unsigned char addr, unsigned char dat)
{
    addr&=0xE0;
    addr|=0x01;
    SynchSerSend(addr);     //Command Write Single Character
    SynchSerSend(dat);      //Data Character
    _wait_ms(10);
}

void K10_WriteString_SS (unsigned char addr, unsigned int *ptr)
{
    unsigned char dat;
    addr&=0xE0;
    addr|=0x02;
    SynchSerSend(addr);     //Command Write Multiple Character

    dat=(*ptr>>8);
    while(dat!=0)
      {
        SynchSerSend(dat);      //Data Character (High byte)
        dat=*ptr;
        if (dat!=0)
          {
            SynchSerSend(dat);  //Data Character (Low byte)
            ptr++;
            dat=(*ptr>>8);
          }
      }
    SynchSerSend(dat);          //Null character
    _wait_ms(10);
}

void K10_LocateDDRAM_SS (unsigned char addr, unsigned char kolom, unsigned char baris)
{
    addr&=0xE0;
    addr|=0x03;
    SynchSerSend(addr);     //Command Locate DDRAM
    SynchSerSend(kolom);    //Data Kolom
    SynchSerSend(baris);    //Data Baris
    _wait_ms(10);
}

void K10_LocateCGRAM_SS (unsigned char addr, unsigned char alamat)
{
    addr&=0xE0;
    addr|=0x04;
    SynchSerSend(addr);     //Command Locate CGRAM
    SynchSerSend(alamat);   //Data Alamat
    _wait_ms(10);
}

void K10_SetContrast_SS (unsigned char addr, unsigned char dat)
{
    addr&=0xE0;
    addr|=0x05;
    SynchSerSend(addr);     //Command Set Contrast
    SynchSerSend(dat);      //Data Character
    _wait_ms(10);
}

unsigned char K10_ReadDDRAMData_SS (unsigned char addr)
{
    unsigned char dat;
    addr&=0xE0;
    addr|=0x06;
    SynchSerSend(addr);     //Command Read Data DDRAM
    _wait_ms(10);
    dat=SynchSerGet();      //Data Character
    _wait_ms(10);
    return dat;
}

unsigned char K10_ReadCGRAMData_SS (unsigned char addr)
{
    unsigned char dat;
    addr&=0xE0;
    addr|=0x07;
    SynchSerSend(addr);     //Command Read Data CGRAM
    _wait_ms(10);
    dat=SynchSerGet();      //Data Character
    _wait_ms(10);
    return dat;
}

unsigned char K10_ReadDDRAMAddr_SS (unsigned char addr)
{
    unsigned char dat;
    addr&=0xE0;
    addr|=0x08;
    SynchSerSend(addr);     //Command Read Address DDRAM
    _wait_ms(10);
    dat=SynchSerGet();      //Data Alamat
    _wait_ms(10);
    return dat;
}

unsigned char K10_ReadCGRAMAddr_SS (unsigned char addr)
{
    unsigned char dat;
    addr&=0xE0;
    addr|=0x09;
    SynchSerSend(addr);     //Command Read Address CGRAM
    _wait_ms(10);
    dat=SynchSerGet();      //Data Alamat
    _wait_ms(10);
    return dat;
}

void K10_CommandLCD_SS (unsigned char addr, unsigned char dat)
{
    addr&=0xE0;
    addr|=0x0A;
    SynchSerSend(addr);     //Command Write Single Character
    SynchSerSend(dat);      //Data Character
    _wait_ms(10);
}

void main (void)
{
    unsigned char data;
    SynchSerReset();
    IE=0x00;
    AddressSS=ADDR_SPC_LCD;
	_wait_ms(1000);

    K10_Control_SS(AddressSS,0x8E);     //BackLight ON, Cursor Shift, No Shift, Increment, Display ON, Cursor ON dan Blink OFF
    K10_SetContrast_SS(AddressSS,0x01);     //Kontras LCD = 07
    K10_CommandLCD_SS(AddressSS,0x01);      //Inst.LCD (Display Clear), DDRAM addr = 00H
    _wait_ms(1000);
    
    K10_WriteSingle_SS(AddressSS,'I');
    K10_WriteSingle_SS(AddressSS,'E');
    
    K10_LocateDDRAM_SS(AddressSS,0x00,0x01);    //Copy character kolom 0, baris 1
    data=K10_ReadDDRAMData_SS(AddressSS);
    K10_LocateDDRAM_SS(AddressSS,14,0x01);      //Paste character kolom 14, baris 0
    K10_WriteSingle_SS(AddressSS,data);
    K10_LocateDDRAM_SS(AddressSS,0x01,0x01);    //Copy character kolom 1, baris 1
    data=K10_ReadDDRAMData_SS(AddressSS);
    K10_LocateDDRAM_SS(AddressSS,15,0x01);      //Paste character kolom 15, baris 0
    K10_WriteSingle_SS(AddressSS,data);

    K10_LocateDDRAM_SS(AddressSS,0x03,0x01);
    K10_WriteString_SS(AddressSS,text1);
    K10_LocateDDRAM_SS(AddressSS,0x00,0x02);
    K10_WriteString_SS(AddressSS,text2);
    
    K10_LocateDDRAM_SS(AddressSS,0x00,0x01);    //Kursor pada kolom 0, baris 1
    _wait_ms(1500);
    K10_Control_SS(AddressSS,0x8F);     //BackLight ON, Cursor Shift, No Shift, Increment, Display ON, Cursor ON dan Blink ON
    _wait_ms(1500);
    K10_Control_SS(AddressSS,0x0E);     //BackLight OFF, Cursor Shift, No Shift, Increment, Display ON, Cursor ON dan Blink OFF
    _wait_ms(1500);
    K10_Control_SS(AddressSS,0x8C);     //BackLight ON, Cursor Shift, No Shift, Increment, Display ON, Cursor OFF dan Blink OFF
    while(1);
}