#include <reg51.h>
#include <stdio.h>

//Hubungan DT-51 LCMS dengan SPC CHARACTER LCD (UART)
//Pin P3.1 (TXD) --- SPC Character LCD Pin 3 (RX)
//Pin P3.0 (RXD) --- SPC Character LCD Pin 2 (TX)

//Text disimpan pada code memory (\x00 = Null, 00H)
code char text1[]={"IE\x00"};
code char text2[]={"DT-51 UART\x00"};
code char text3[]={"SPC CharacterLCD\x00"};

void init232(void)
{
    TMOD=(TMOD&0x0F)|0x20;  //Timer 1 Mode 2 (Auto reload)
    SCON=0x52;              //Serial Mode 1 (8-bit UART), REN=1 (Receiver Enable)
                            // TI=1 (Transmit buffer empty)
    PCON=PCON&0x7F;         //SMOD=0 (Double baud rate disable)
    TH1=0xFD;               //9600 bps (11,059 MHz)
    ET1=0;                  //Disable Timer 1 interrupt
    TR1=1;                  //Timer 1 Run
}

void sendchar (unsigned char dat)
{
    while (TI==0);
    TI=0;
    SBUF=dat;           //Send a character
}

unsigned char receivechar (void)
{
    while (RI==0);
    RI=0;
    return SBUF;        //Receive a character
}

void K10_Control_UART (unsigned char dat)
{
    sendchar(0xFE);         //Command
    sendchar(0x00);         //Command Control
    sendchar(dat);          //Data Control
    _wait_ms(1);
}

//WriteSingle dengan menggunakan Command Write Multiple Character
void K10_WriteSingle_UART (unsigned char dat)
{
    sendchar(0xFE);         //Command
    sendchar(0x02);         //Command Write Multiple Character
    sendchar(dat);          //Data Character
    sendchar(0x00);          //Null character
    _wait_ms(1);
}

void K10_WriteString_UART (unsigned int *ptr)
{
    unsigned char dat;
    sendchar(0xFE);         //Command
    sendchar(0x02);         //Command Write Multiple Character
    dat=(*ptr>>8);
    while(dat!=0)
      {
        sendchar(dat);      //Data Character (High byte)
        dat=*ptr;
        if (dat!=0)
          {
            sendchar(dat);  //Data Character (Low byte)
            ptr++;
            dat=(*ptr>>8);
          }
      }
    sendchar(dat);          //Null character
    _wait_ms(1);
}

void K10_LocateDDRAM_UART (unsigned char kolom, unsigned char baris)
{
    sendchar(0xFE);         //Command
    sendchar(0x03);         //Command Locate DDRAM
    sendchar(kolom);        //Data Kolom
    sendchar(baris);        //Data Baris
    _wait_ms(2);
}

void K10_LocateCGRAM_UART (unsigned char alamat)
{
    sendchar(0xFE);         //Command
    sendchar(0x04);         //Command Locate CGRAM
    sendchar(alamat);       //Data Alamat
    _wait_ms(2);
}

void K10_SetContrast_UART (unsigned char dat)
{
    sendchar(0xFE);         //Command
    sendchar(0x05);         //Command Set Contrast
    sendchar(dat);          //Data Character
    _wait_ms(1);
}

unsigned char K10_ReadDDRAMData_UART (void)
{
    unsigned char dat;
    sendchar(0xFE);         //Command
    sendchar(0x06);         //Command Read Data DDRAM
    dat=receivechar();      //Data Character
    _wait_ms(1);
    return dat;
}

unsigned char K10_ReadCGRAMData_UART (void)
{
    unsigned char dat;
    sendchar(0xFE);         //Command
    sendchar(0x07);         //Command Read Data CGRAM
    dat=receivechar();      //Data Character
    _wait_ms(1);
    return dat;
}

unsigned char K10_ReadDDRAMAddr_UART (void)
{
    unsigned char dat;
    sendchar(0xFE);         //Command
    sendchar(0x08);         //Command Read Address DDRAM
    dat=receivechar();      //Data Alamat
    _wait_ms(1);
    return dat;
}

unsigned char K10_ReadCGRAMAddr_UART (void)
{
    unsigned char dat;
    sendchar(0xFE);         //Command
    sendchar(0x09);         //Command Read Address CGRAM
    dat=receivechar();      //Data Alamat
    _wait_ms(1);
    return dat;
}

void K10_CommandLCD_UART (unsigned char dat)
{
    sendchar(0xFE);         //Command
    sendchar(0x0A);         //Command Write Single Character
    sendchar(dat);          //Data Character
    _wait_ms(1);
}

void main (void)
{
    unsigned char data;
    _wait_ms(1000);
    IE=0x00;
    init232();
    K10_Control_UART(0x8E);         //BackLight ON, Cursor Shift, No Shift, Increment, Display ON, Cursor ON dan Blink OFF
    K10_SetContrast_UART(0x01);     //Kontras LCD = 07
    K10_CommandLCD_UART(0x01);      //Inst.LCD (Display Clear), DDRAM addr = 00H
    _wait_ms(1000);
    
    K10_WriteString_UART(text1);
    
    K10_LocateDDRAM_UART(0x00,0x01);    //Copy character kolom 0, baris 0
    data=K10_ReadDDRAMData_UART();
    K10_LocateDDRAM_UART(14,0x01);      //Paste character kolom 14, baris 0
    K10_WriteSingle_UART(data);
    K10_LocateDDRAM_UART(0x01,0x01);    //Copy character kolom 1, baris 0
    data=K10_ReadDDRAMData_UART();
    K10_LocateDDRAM_UART(15,0x01);      //Paste character kolom 15, baris 0
    K10_WriteSingle_UART(data);

    K10_LocateDDRAM_UART(0x03,0x01);
    K10_WriteString_UART(text2);
    K10_LocateDDRAM_UART(0x00,0x02);
    K10_WriteString_UART(text3);
    
    K10_LocateDDRAM_UART(0x00,0x01);    //Kursor pada kolom 0, baris 1
    _wait_ms(1500);
    K10_Control_UART(0x8F);         //BackLight ON, Cursor Shift, No Shift, Increment, Display ON, Cursor ON dan Blink ON
    _wait_ms(1500);
    K10_Control_UART(0x0E);         //BackLight OFF, Cursor Shift, No Shift, Increment, Display ON, Cursor ON dan Blink OFF
    _wait_ms(1500);
    K10_Control_UART(0x8C);         //BackLight ON, Cursor Shift, No Shift, Increment, Display ON, Cursor OFF dan Blink OFF
    while(1);
}